@@ -24,7 +24,8 @@ from python_kemptech_api.exceptions import (
     BackupFailed,
     DownloadUserCertFailed,
     UnauthorizedAccessError,
-    UserAlreadyExistsException)
+    UserAlreadyExistsException,
+    NotVirtualServiceInstanceError)
 from python_kemptech_api.capabilities import CAPABILITIES, DEFAULT
 from .utils import (
     validate_port,
@@ -232,6 +233,32 @@ class LoadMaster(HttpClient):
         curl.append(command)
         return curl
 
+    def clone_virtual_service(self, service, ip, port, protocol, enable=True,
+                              dry_run=False):
+        """Given a VirtualService instance, add it to this LoadMaster
+
+        :param service: The VirtualService instance to clone
+        :param ip: The new IP address of the virtual service
+        :param port: The new port of the virtual service
+        :param protocol: The new protocol of the virtual service
+        :param enable: Enable the VirtualService
+        :param dry_run: Don't save the VirtualSerivce immediately
+        :return: The altered VirtualService tied to the this LoadMaster
+        """
+        if not isinstance(service, VirtualService):
+            raise NotVirtualServiceInstanceError()
+        service.endpoint = self.endpoint
+        service.ip_address = self.ip_address
+        service.cert = self.cert
+        service.vs = ip
+        service.port = port
+        service.prot = protocol
+        service.enable = get_api_bool_string(enable)
+        service.index = None
+        if not dry_run:
+            service.save()
+        return service
+
     def create_virtual_service(self, ip, port=80, protocol="tcp"):
         """VirtualService factory with pre-configured LoadMaster connection."""
         vs = VirtualService(self.access_info, ip, port, protocol)
@@ -275,17 +302,11 @@ class LoadMaster(HttpClient):
 
     def get_all_objects(self):
         # x variables are the object while x_data is the OrderedDict
-        global_real_servers_data = []
         virtual_services = []
         response = self._get("/listvs")
         data = get_data(response)
         virtual_services_data = data.get('VS', [])
         virtual_services_data = cast_to_list(virtual_services_data)
-        # loop through all vss and gather real server objects
-        for service_data in virtual_services_data:
-            real_servers_data = service_data.get('Rs', [])
-            real_servers_data = cast_to_list(real_servers_data)
-            global_real_servers_data.extend(real_servers_data)
 
         # create vs and rs objects at this point
         # loop through all vss and attach matching real server objects
@@ -298,27 +319,12 @@ class LoadMaster(HttpClient):
                                                                vs)
             else:
                 virt_serv = self.build_virtual_service(service_data, response)
-            real_servers_x = cast_to_list(service_data.get("Rs", []))
-            for server_data in real_servers_x:
+            real_servers = cast_to_list(service_data.get("Rs", []))
+            for server_data in real_servers:
                 rs = virt_serv.build_real_server(server_data)
                 virt_serv.real_servers.append(rs)
             virtual_services.append(virt_serv)
 
-        """
-        rs_vs_indexes = [(rs['VSIndex'], rs)
-                         for rs in global_real_servers_data]
-        rs_vs_indexes = dict(rs_vs_indexes)
-
-        for vs in virtual_services:
-            if vs.index in rs_vs_indexes:
-                rs = vs.build_real_server(rs_vs_indexes[vs.index])
-                vs.real_servers.append(rs)
-            for subvs in vs.subvs_entries:
-                if subvs.index in rs_vs_indexes:
-                    rs = subvs.build_real_server(rs_vs_indexes[subvs.index])
-                    subvs.real_servers.append(rs)
-        """
-
         return virtual_services
 
     def build_virtual_service(self, service, response=None):
@@ -626,7 +632,8 @@ class KempBaseObjectModel(object):
     # blacklist attributes that shouldn't be pushed to the loadmaster.
     _API_IGNORE = (
         "ip_address", "endpoint", "rsindex", "vsindex", "index", "status",
-        "subvs_data", "subvs_entries", "real_servers", "cert",
+        "subvs_data", "subvs_entries", "real_servers", "cert", "rsminimum",
+        "checkuse1_1"
     )
 
     def __repr__(self):
@@ -746,6 +753,15 @@ class VirtualService(HttpClient, KempBaseObjectModel):
                 "vs": self.index,
             }
 
+    @property
+    def checkuse1_1(self):
+        """This property exists because . can not be in a variable name"""
+        return self.__dict__.get('checkuse1.1', None)
+
+    @checkuse1_1.setter
+    def checkuse1_1(self, value):
+        self.__dict__['checkuse1.1'] = value
+
     def save(self, update=False):
         """ Send API request to add or update the VirtualService
 
@@ -818,6 +834,25 @@ class VirtualService(HttpClient, KempBaseObjectModel):
         self.qos = service.get('QoS', None)
         self.idletime = service.get('Idletime', None)
 
+        # Real servers section
+        self.checktype = service.get('CheckType', None)
+        self.checkhost = service.get('CheckHost', None)
+        self.checkpattern = service.get('CheckPattern', None)
+        self.checkurl = service.get('CheckUrl', None)
+        self.checkcodes = service.get('CheckCodes', None)
+        self.checkheaders = service.get('CheckHeaders', None)
+        self.matchlen = service.get('MatchLen', None)
+        self.checkuse1_1 = service.get('CheckUse1.1', None)
+        self.checkport = falsey_to_none(int(service.get('CheckPort', 0)))
+        self.checkuseget = service.get('CheckUseGet', None)
+        self.extrahdrkey = service.get('ExtraHdrKey', None)
+        self.extrahdrvalue = service.get('ExtraHdrValue', None)
+        self.checkpostdata = service.get('CheckPostData', None)
+        self.rsruleprecedence = service.get('RSRulePrecedence', None)
+        self.rsruleprecedencepos = service.get('RSRulePrecedencePos', None)
+        self.enhancedhealthchecks = service.get('EnhancedHealthchecks', None)
+        self.rsminimum = service.get('RsMinimum', None)
+
     def create_sub_virtual_service(self):
         """VirtualService factory with pre-configured LoadMaster connection
 
